import logging
import os

from notedrive.sqlalchemy.base import BaseTable, meta
from notetool.secret import read_secret
from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base

uri = read_secret(cate1='notecoin', cate2='dataset', cate3='db_path')
uri = uri or f'sqlite:///{os.path.abspath(os.path.dirname(__file__))}/data/notecoin.db'

engine = create_engine(uri)
Base = declarative_base()
logging.info(f'uri:{uri}')

__all__ = ['meta', 'BaseTable']

#
# class BaseTable:
#     def __init__(self, table_name, *args, **kwargs):
#         self.table_name = table_name
#         self.table: Table = None
#         meta.create_all(engine)
#
#     def insert(self, values, keys=None, *args, **kwargs):
#         meta.create_all(engine)
#         cols = [col.name for col in self.table.columns]
#         if isinstance(values, dict):
#             values = dict([(k, v) for k, v in values.items() if k in cols])
#         elif isinstance(values, list):
#             if isinstance(values[0], dict):
#                 values = [dict([(k, v) for k, v in item.items() if k in cols]) for item in values]
#             elif isinstance(values[0], list):
#                 values = [dict(zip(keys, item)) for item in values]
#
#         with warnings.catch_warnings():
#             warnings.simplefilter("ignore")
#             # code here...
#
#             if str(engine.url).startswith('sqlite'):
#                 ins = self.table.insert(values=values).prefix_with("OR IGNORE")
#             else:
#                 ins = self.table.insert(values=values).prefix_with("IGNORE")
#             engine.execute(ins)



import json
import time

import ccxt
import pandas as pd
from ccxt.base.exchange import Exchange
from notecoin.base.database import KlineData
from tqdm import tqdm


class LoadDataKline:
    def __init__(self, exchange: Exchange, *args, **kwargs):
        self.table = KlineData()
        self.exchange = exchange
        super(LoadDataKline, self).__init__(*args, **kwargs)

    def load_all(self, *args, **kwargs):
        self.exchange.load_markets()
        for sym in tqdm(self.exchange.symbols):
            if ':' not in sym:
                self.load(sym, *args, **kwargs)

    def load(self, symbol, timeframe='1m', max_retries=3, limit=100, *args, **kwargs):
        max_time, min_time = self.table.select_symbol_maxmin(symbol)
        timeframe_duration_in_ms = self.exchange.parse_timeframe(timeframe) * 1000
        timedelta = limit * timeframe_duration_in_ms

        if max_time == 0 or min_time == 0:
            max_time = self.exchange.milliseconds()
            min_time = self.exchange.milliseconds()

        def load(fetch_since):
            result = self.exchange.fetch_ohlcv(symbol, timeframe, fetch_since, limit=limit)
            result = self.exchange.sort_by(result, 0)
            if len(result) == 0:
                return False, f'result is empty, fetch_since:{fetch_since}'
            pbar.set_postfix({'fetch_since': self.exchange.iso8601(result[0][0])})
            df = pd.DataFrame(result, columns=['timestamp', 'open', 'close', 'low', 'high', 'vol'])
            df['symbol'] = symbol
            self.table.insert(json.loads(df.to_json(orient='records')))
            time.sleep(int(self.exchange.rateLimit / 1000))
            return True, result

        earliest_timestamp = min_time
        pbar = tqdm(range(min_time, min_time-100*timedelta, -timedelta), desc=symbol)
        for _ in pbar:
            status, result = load(earliest_timestamp - timedelta)
            if status is False:
                print(result)
                break
            earliest_timestamp = result[0][0]

        #earliest_timestamp = self.exchange.milliseconds()
        lasted_timestamp = max_time
        pbar = tqdm(range(max_time, max_time+100*timedelta, timedelta), desc=symbol)
        for _ in pbar:
            status, result = load(lasted_timestamp)
            if status is False:
                print(result)
                break
            lasted_timestamp = result[-1][0]



import uvicorn
from fastapi import FastAPI
from notecoin.okex.server.const import (account_account, base_api,
                                        market_tickers, websocket_api)
from notecoin.strategy.sell_strategy import AutoSeller

seller = AutoSeller()

app = FastAPI()
app.include_router(account_account)
app.include_router(market_tickers)
app.include_router(base_api)
app.include_router(seller)
app.include_router(websocket_api)

# uvicorn notecoin_server:app --host '0.0.0.0' --port 8444 --reload
# uvicorn notecoin_server:app --host '0.0.0.0' --port 8444
# uvicorn notecoin_server: app - -host '0.0.0.0' - -port 8444

uvicorn.run(app, host='0.0.0.0', port=8444)
